/**
 * Meditation specific functions
 *
 * TODO: Handling feedback : entering question on website, displaying in app, returning values to website, displaying values on website
 * TODO: Handling statistics : counting full plays in app, returning values to website, displaying values on website
 * TODO: Check localStorage is correct when app started up - clear() and rebuild each startup?
 * TODO: Could also take the opportunity to delete old audios that have not been played in a while or restrict maximum cache size
 * TODO: Check date handling is correct - GMT in datafile, but use local TMZ and display options for phone
 * TODO: Show the number of new items within each category (up to 99, then show 99+ ?)
 * TODO: Mark an unread item as new
 *
 * OPTION: Allow sending feedback more than once for an item?
 */

//var meditationURL = 'http://www.codejuggler.com/clients/Meditation/feed/app';
//var meditationURL = 'http://192.168.0.8/WebDocs/meditation-2015-09-22/wp/?feed=app';
var meditationURL = 'http://meditationlounge.org/wp/?feed=app';
var meditationData = null;

var fs = null; // Reference to local file system (class FileSystem)
var remoteBase = ''; // Base path for remote files (class string)
var localBase = 'cordova.file.cacheDirectory'; // Base path for local files (class string)
var baseDir;

var backPage = null; // Reference to the page to go back to via hardware backbutton
var meditationDataFile = null; // Reference to the meditation data file (class File)
var interval = 24 * 60 * 60 * 1000; // wait a day between checks
// var interval = 60 * 1000; // TESTING wait a minute between checks

jQuery(document).ready(
    function () {
        console.log('jQuery is ready');

        document.addEventListener('deviceready', onDeviceReady, {passive:false});
    }
    );


jQuery(document).on('mobileinit',
    function () {
        console.log('jQuery Mobile is ready');

        jQuery.mobile.defaultPageTransition = 'slide'; 	// Slide from one page to the next rather than the default fade
        jQuery.mobile.transitionFallbacks.slide = 'none'; // If you can't slide, then just jump, don't fade (the normal fallback).
        jQuery.mobile.touchOverflowEnabled = true; // Enable smooth scrolling in iOS
        jQuery.mobile.dialog.prototype.options.closeBtnText = 'close'; // hidden screenreader close button text on dialogues
        jQuery(document).on('swiperight', onBackButton);        
    }
);


function onDeviceReady() {

    if (window.cordova.logger) {
        window.cordova.logger.__onDeviceReady();
    }

    console.log('Device is ready');

    document.addEventListener('resume', onResume, {passive: false});

    handleExternalURLs();

    if (device.platform == 'Android') {
        $('body').addClass('android'); // hide the back button on Android
        document.addEventListener('backbutton', onBackButton, {passive: false});
        // localBase = 'Android/data/com.codejuggler.meditationLounge';
    }

    if (device.platform == 'iOS')
    {
		StatusBar.overlaysWebView(false);
    //	localBase = '../Library/Caches/meditationLounge';
    }
    
    $('#refresh-link').click(function(){onResume(true)});

    window.requestFileSystem(LocalFileSystem.PERSISTENT, 0, onValidFileSystem, onErrorHandler);
}


// We have a valid file system, find the path to the right place - cached, writable, non-iCloud
function onValidFileSystem(newFS) {
    fs = newFS;

    // console.log('File system root name ' + fs.root.name);
    window.resolveLocalFileSystemURL(cordova.file.cacheDirectory, onPathOK, onErrorHandler);
}


// Now create a sub directory
function onPathOK(_pathDir) {
    // console.log('Directory ' + _pathDir.toURL());
    _pathDir.getDirectory('meditationlounge', { create: true, exclusive: false }, onDirOK, onErrorHandler);
}


// Directory added OK if not already there, so download the json data to it if we can
function onDirOK(_baseDir) {
    baseDir = _baseDir;
    // console.log('Dir created : ' + baseDir.toURL());

    // Get local data if possible, otherwise get remote data

    var lastCheck = window.localStorage.getItem('lastcheck');
    var timeNow = new Date().getTime();

    console.log('Checking in  ' + ((interval - (timeNow - lastCheck)) / 1000) + ' seconds');

    if (isNetworkConnected() && (lastCheck === null || (timeNow - lastCheck) > interval)) {
        onResume(true);
    } else {
        getLocalData();
    }
}


// App has been brought back from the background.
// Only retry the json file if we are online, and either haven't checked lately, or if checkNow is true
// checkNow is true if the reload button is pressed
function onResume(checkNow) {
    var lastCheck = window.localStorage.getItem('lastcheck');
    var timeNow = new Date().getTime();

    console.log('Checking in  ' + ((interval - (timeNow - lastCheck)) / 1000) + ' seconds');

    if (isNetworkConnected() && (checkNow === true || lastCheck === null || (timeNow - lastCheck) > interval)) {
        console.log('Resumed - with good connection, try remote data');
        var fileTransfer = new FileTransfer();
        var uri = encodeURI(meditationURL);
        var fileName = baseDir.toURL() + '/meditationData.json';
        fileTransfer.download(uri, fileName, gotDataFileEntry, onErrorHandler);
        $('#refresh-image').attr('src', 'images/ajax-loader32.gif');
        var ms = new Date();
        console.log('Last check ' + ms.toLocaleString());
        window.localStorage.setItem('lastcheck', ms.getTime());
    }
}


// Get the previously downloaded data
function getLocalData() {
    console.log('Getting local data');
    baseDir.getFile('meditationData.json', null, gotDataFileEntry, onErrorHandler);
}


// Get the File object from the FileEntry object
function gotDataFileEntry(fileEntry) {
    // console.log('Data File Entry found ' + fileEntry.toURL());
    fileEntry.file(gotDataFile, onErrorHandler);
}


// Read the JSON from the File object
function gotDataFile(file) {
    console.log('JSON file downloaded OK - ' + file.name + ' ' + file.size + ' bytes');

    meditationDataFile = file;

    var reader = new FileReader();
    reader.onload = function (evt) {
        console.log('loading done OK');
        meditationData = JSON.parse(evt.target.result);
        remoteBase = meditationData.global.base;
        loadAllFiles();
        $('#refresh-image').attr('src', 'images/refresh-icon-static32.png');
    };

    reader.onerror = function (evt) {
        console.log('loading done NOT OK');
        onErrorHandler(evt);
    };

    reader.onloaded = function (evt) {
        console.log('loading done');
    };

    reader.readAsText(meditationDataFile);
}


// For each of the three sections, load in all images
function loadAllFiles() {
    if (meditationData) {
        if (meditationData.armchairmeditations) loadSectionFiles(meditationData.armchairmeditations);
        if (meditationData.armchairwisdom) loadSectionFiles(meditationData.armchairwisdom);
        if (meditationData.thoughtfortheday) loadSectionFiles(meditationData.thoughtfortheday);
        if (meditationData.armchairbooks) loadSectionFiles(meditationData.armchairbooks);
    }
}


// For a single section, load all the images
function loadSectionFiles(data) {
    for (var category in data) {
        if (data[category].image) loadSingleFile(data[category].image, null);
        for (var item in data[category].items) {
            if (data[category].items[item].image) loadSingleFile(data[category].items[item].image, null);
        }
    }
}


// Load a single file from a url if it is not already local
function loadSingleFile(fileURL, onProgressFn, id) {
    var shortFileName = extractFilename(fileURL);
    var fileName = baseDir.toURL() + shortFileName;

    if (window.localStorage.getItem(shortFileName) === null) {
        console.log('File is not local, downloding from ' + fileURL);

        var fileTransfer = new FileTransfer();
        fileTransfer.onprogress = onProgressFn;
        var uri = encodeURI(fileURL);

        var gotImageFileEntry = function (fileEntry) {
            // console.log('File downloaded to ' + fileEntry.fullPath + ' ' + fileEntry.toURL() + ' ' + baseDir.fullPath);
            fileEntry.file(function (fileObj) {
                if (fileObj.name.substr(-3, 3) == 'mp3') {
                    if (id === null) // audio file is a meditation
                    {
                        $('#audio-downloaded').html('remove meditation (' + toMb(fileObj.size) + 'Mb)');
                        //$('#audio_clear').attr('href', 'javascript:clearAudio("' + fileEntry.fullPath + '")');
                        $('#audio_clear').click(function(){console.log('clearing audio ' + fileEntry.fullPath);clearAudio(fileEntry.fullPath)});
                        $('#audio_duration').empty();
                    }
                    else // audio file is a book chapter
                    {
                        totalBookSize = totalBookSize + Number(fileObj.size);
                        bookList.push(shortFileName);
                        $('#audio-downloaded').html('remove all chapters (' + toMb(totalBookSize) + 'Mb)');
                        //$(id).html(' (' + toMb(fileObj.size) + 'Mb)');
                        $('#book-download-' + id).empty();
                        $('#book-icon-' + id).html('<img src="images/play-icon.png">');
                    }

                    if (!myMedia) myMedia = new Media(fileEntry.toInternalURL(), completedAudio, errorAudio);
                    window.localStorage.setItem(shortFileName + '-size', fileObj.size);
                }
            }, onErrorHandler);
            var path = fileEntry.toURL(); 
            if (device.platform == 'iOS') {path = window.WkWebView.convertFilePath(fileEntry.toURL())};
            // console.log('Now downloaded ' + path );
            window.localStorage.setItem(shortFileName, path); // mark as downloaded
        };

        fileTransfer.download(uri, fileName, gotImageFileEntry, onErrorHandler);
    }
    else {
        // console.log('Already downloaded, checking ' + fileName);
        baseDir.getFile(shortFileName, null,
            function (fileEntry) {
                var path = fileEntry.toURL();
                if (device.platform == 'iOS') {path = window.WkWebView.convertFilePath(fileEntry.toURL())};                
                // console.log('Already downloaded ' + path);
                window.localStorage.setItem(shortFileName, path);
            },
            function () {
                //console.log('File missing - mark as such for next sync');
                window.localStorage.removeItem(shortFileName);
            }
            );
    }
}


// Get an audio file - show progress as it downloads
function getAudio(fileURL, id) {
    // console.log('Loading audio ' + fileURL);

    var onProgress = function (progressEvent) {
        if (progressEvent.lengthComputable) {
            var percent = (progressEvent.loaded / progressEvent.total * 100).toFixed(0);

            if (id === null) // meditation
            {
                //$('#audio-downloaded').html(' downloading (' + toMb(progressEvent.loaded) + '/' + toMb(progressEvent.total) + 'Mb)');
                $('#audio-downloaded').html(' downloading (' + percent + '%)');
            }
            else // book chapter
            {
                //$('#book-download-'+id).html(' (' + toMb(progressEvent.loaded) + '/' + toMb(progressEvent.total) + 'Mb)');
                
                $('#book-download-' + id).html(' (' + percent + '%)');
            }
        }
    };

    loadSingleFile(fileURL, onProgress, id);
}


// Return the last section of a url or filepath, will always have a '/' at the start
function extractFilename(filePath) {
    var index = filePath.lastIndexOf('/');
    if (index == -1)
        return filePath;
    else
        return filePath.slice(index + 1);
}


// Return true if there is any connection - wifi, 2g, 3g etc, false otherwise
function isNetworkConnected() {
    var networkState = navigator.connection.type;
    return (networkState != Connection.UNKNOWN && networkState != Connection.NONE);
}


// Return true if there is a wifi or ethernet connection, false otherwise
function isNetworkFast() {
    var networkState = navigator.connection.type;
    return (networkState == Connection.WIFI || networkState == Connection.ETHERNET);
}


// Back button handler to stop music and go back to correct section
function onBackButton(event) {
    //console.log('back called with ' + JSON.stringify(event));
    
    if (event && event.type == 'swiperight' && backPage === null) return; // ignore top level swipe
    
    if (backPage === null)
        navigator.app.exitApp();
    else {
        console.log('Back key pressed - go to ' + backPage);
        //$.mobile.changePage(backPage, { reverse: 'true'} );
        $(':mobile-pagecontainer').pagecontainer('change', backPage, { reverse: 'true' });
        oldNum = -1;
    }
}


// Called when entering any section
function stopAllAudio() {
    if (myMedia) {
        console.log('stopping and releasing audio');
        myMedia.stop();
        myMedia.release();
        if (device.platform == 'iOS') window.plugins.insomnia.allowSleepAgain(); // releaseScreenLock();
        playing = false;
        clearInterval(mediaTimer);
    }

    var videoplayer = document.getElementById('videoplayer');

    //if (videoplayer) 
    //{
    //    console.log('stopping video');
    //    videoplayer.stopVideo();
    //}
}

// Called when we cannot find a file system.
// Or we cannot read a local file
// Need to exit gracefully from the app?
function onErrorHandler(e) {
    var msg = '';

    switch (e.code) {
        case FileError.NOT_FOUND_ERR:
            msg = 'NOT_FOUND_ERR';
            break;
        case FileError.SECURITY_ERR:
            msg = 'SECURITY_ERR';
            break;
        case FileError.ABORT_ERR:
            msg = 'ABORT_ERR';
            navigator.notification.alert('Please connect to the internet and try again', null, 'No network found', 'OK');
            break;
        case FileError.NOT_READABLE_ERR:
            msg = 'NOT_READABLE_ERR';
            break;
        case FileError.ENCODING_ERR:
            msg = 'ENCODING_ERR';
            break;
        case FileError.NO_MODIFICATION_ALLOWED_ERR:
            msg = 'NO_MODIFICATION_ALLOWED_ERR';
            break;
        case FileError.INVALID_STATE_ERR:
            msg = 'INVALID_STATE_ERR';
            break;
        case FileError.SYNTAX_ERR:
            msg = 'SYNTAX_ERR';
            break;
        case FileError.INVALID_MODIFICATION_ERR:
            msg = 'INVALID_MODIFICATION_ERR';
            break;
        case FileError.QUOTA_EXCEEDED_ERR:
            msg = 'QUOTA_EXCEEDED_ERR';
            break;
        case FileError.TYPE_MISMATCH_ERR:
            msg = 'TYPE_MISMATCH_ERR';
            break;
        case FileError.PATH_EXISTS_ERR:
            msg = 'PATH_EXISTS_ERR';
            break;
        default:
            msg = 'Unknown Error (' + e.code + ')';
            break;
    }
    console.log('Error: ' + msg);
}


/* Storing the scroll pos of the previous page - doesn't really work.
var scrollPos = null;

$(document).on('pageshow', '#category', function() {
    if(scrollPos !== null) {
        $.mobile.silentScroll(scrollPos);
        console.log('Restore Scroll ' + scrollPos);
    }
});


$(document).on('pagebeforeshow', '#item', function() {
    scrollPos =  $('#item').offset().top;
	console.log('Saved Scroll ' + scrollPos);
});
*/


// Create a list of categories within a section - currently only used by armchair-meditations
function showCategories(section, options) {
    stopAllAudio();

    backPage = '#menu';
    var $page = $('#section');
    var $header = $page.children(':jqmData(role=header)');
    var $content = $page.children(':jqmData(role=content)');

    var categoriesObj = meditationData[section];

    // var markup = '<div class="outer ui-grid-b ui-responsive">';
    var markup = '<div class="detailOuter">';

    var gridLetter = 0;
    var grid = ['a', 'b', 'c'];

    for (var category in categoriesObj) {
        // console.log(category);
        var categoryObj = categoriesObj[category];
        // markup += '<div class="category"><a href="#category_' + section + '_' +  category + '"><img src="'+ remoteBase + categoryObj.image + '"></a></div>';

        var shortFileName = extractFilename(categoryObj.image);
        var fullPath = window.localStorage.getItem(shortFileName);
        if (fullPath === null) fullPath = 'images/default.png'; // image not downloaded yet, use a local placeholder
        markup += '<div class="detailInner"><a href="#category_' + section + '_' + category + '"><img class="thumbnail" src="' + fullPath + '"></a></div>';
    }
    markup += '</div>';
    
    $content.html(markup);

    // TODO: Use a title from the JSON file
    $header.find('h1').html('meditations');

    $page.page();
    options.dataUrl = '#section_' + section;
    $.mobile.changePage($page, options);
}


// Create a list of items within a category for a section
function showSingleCategory(section, category, options) {
    stopAllAudio();

    console.log('Creating category list for :' + section + ", " + category);

    var $page = $('#category');
    var $header = $page.children(':jqmData(role=header)');
    var $content = $page.children(':jqmData(role=content)');

    var categoryObj = meditationData[section][category];
    var locale = window.navigator.userLanguage || window.navigator.language;
    var markup = '<div class="detailOuter">';
    var i = 0;
    for (var item in categoryObj['items']) {
        var meditation = categoryObj['items'][item];
        var date = new Date(meditation.date);

        var shortFileName = extractFilename(meditation.image);
        var fullPath = window.localStorage.getItem(shortFileName);
        if (fullPath === null) fullPath = "images/default.png"; // image not downloaded yet, use a local placeholder
        // markup += '<div class="detailCategory"><table><tr><td class="items"><a href="#item_' + section + "_" + category + '_' + i + '"><img class="thumbnail" src="' + fullPath + '"></a></td><td class="itemInfo"><a href="#item_' + section + "_" + category + '_' + i + '"><p>' + date.toLocaleDateString(locale) + '<br/><strong>' + meditation.name + '</strong></p></a></td></tr><tr><td colspan="2"><img src="images/dots.png"></td></tr></table></div>';
        markup += '<div class="detailCategory"><a href="#item_' + section + "_" + category + '_' + i + '"><table><tr><td class="items"><img class="thumbnail" src="' + fullPath + '"></a></td><td class="itemInfo"><p>' + date.toLocaleDateString(locale) + '<br/><strong>' + meditation.name + '</strong></p></td></tr><tr><td colspan="2"><img src="images/dots.png"></td></tr></table></a></div>';
        i++;
    }
    markup += '</div>';

    $content.html(markup);

    $header.find('h1').html(categoryObj.title);

    // if there is only one category, then go back to the main menu, otherwise go back to the previous section.
    var length = 0;
    for (var i in meditationData[section]) {
        length++;
    }
    var backMenu = (length <= 1) ? '#menu' : '#section_' + section;
    backPage = backMenu;
    console.log('Go back to :' + backPage);
    $header.find('a').attr('href', backPage);

    $page.page();
    options.dataUrl = '#category_' + section + '_' + category;
    $.mobile.changePage($page, options);
}


// Create a page for a single item within a category for a section
// Add support for book_meditations (category_book_meditations)
function showItem(section, category, item, options) {
    stopAllAudio();

    console.log('Creating item  for :' + section + ", " + category + ", " + item);

    var $page = $('#item');
    var $header = $page.children(':jqmData(role=header)');
    var $content = $page.children(':jqmData(role=content)');

    var categoryObj = meditationData[section][category];
    var itemObj = categoryObj["items"][item];

    var clearAudioStr = '';
    var audio = '';
    myMedia = null;
    if (itemObj.audio && !Array.isArray(itemObj.audio)) {
        console.log('a single audio');

        var shortFileName = extractFilename(itemObj.audio);
        var fullPath = window.localStorage.getItem(shortFileName);

        if (fullPath === null) {
            getAudio(remoteBase + '/' + shortFileName, null);
            clearAudioStr = '<div class="feedbacktitle"><p><a id="audio_clear" data-transition="none" href="#"><img src="images/rateicon.png"> <span id="audio-downloaded">downloading...</span></a></p></div>';
        }
        else {
            // Check it really is there and download if not
            baseDir.getFile(shortFileName, null,
                function (fileEntry) {
                    console.log('yes, audio OK ' + fileEntry.name);
                    myMedia = new Media(fileEntry.toInternalURL(), completedAudio, errorAudio);
                },
                function () {
                    console.log('Audio file missing - mark as such and reload');
                    window.localStorage.removeItem(shortFileName);
                    getAudio(remoteBase + '/' + shortFileName, null);
                }
                );

            var size = window.localStorage.getItem(shortFileName + "-size");
            var filesize = toMb(size);
            // clearAudioStr = '<div class="feedbacktitle"><p><a id="audio_clear" data-transition="none" href="javascript:clearAudio(\'' + fullPath + '\')"><img src="images/rateicon.png"> <span id="audio-downloaded">remove meditation (' + filesize + 'Mb)</span></a></p></div>';
            clearAudioStr = '<div class="feedbacktitle"><p><a id="audio_clear" data-transition="none" href="#"><img src="images/rateicon.png"> <span id="audio-downloaded">remove meditation (' + filesize + 'Mb)</span></a></p></div>';
        }
        console.log('playing the file ' + fullPath);
    }

        
    // FEEDBACK REMOVED FOR NOW
    // var feedback = '<div class="feedbacktitle"><a data-rel="dialog" data-transition="none" href="#feedback_' + section + "_" + category + "_" + item + '"><img src="images/rateicon.png"> rate & improve</a></div>';
    //

    var image = getImageMarkup(itemObj);
    var video = getVideoMarkup(itemObj);
    var book = getBookMarkup(itemObj);
    var info = getInformationMarkup(itemObj);
    var mediacontrols = getMediaControlMarkup(itemObj);

    // FEEDBACK REMOVED FOR NOW
    //if (window.localStorage.getItem(itemObj.slug + '-feedback-given') == null) {
    //	details += feedback;
    //}

    var markup = '<div class="not-outer">';
    markup += '<div class="meditation ui-grid-a">';

    markup += image;

    markup += '<div class="ui-block-b">';
    markup += video;
    markup += info;
    markup += audio;
    markup += book;
    markup += mediacontrols;
    markup += clearAudioStr;

    markup += '</div>'; // end of ui-block-b

    markup += '</div>'; // end of ui-grid-a

    markup += '</div>'; // end of not-outer
    
    $content.html(markup);

    if (itemObj.audio) {
        $('#play').click(playAudio);
        $('#stop').click(stopAudio);
        if (fullPath != null) {   
            $('#audio_clear').click(function(){
                 clearAudio(fullPath);
            });
        }
    }

    // It's a book
    if (itemObj.audio && Array.isArray(itemObj.audio)) {
        setBookDownloads(itemObj);

        var audios = itemObj.audio;
        for (let i = 0; i < audios.length; i++) {
            console.log('adding book click ' + audios[i].desc);
            $('#book-playing-' + i).click(function() {
                console.log('play audio ' + audios[i].url);
                chooseChapterAudio(audios[i].desc,audios[i].url,i);          
            });
        }

        $('#audio_clear').click(function(){
            clearBookAudio(itemObj.audio);
       });
    }

    $header.find('h1').html(categoryObj.title);

    backPage = '#category_' + section + '_' + category;
    console.log('Go back to :' + backPage);
    $header.find('a').attr('href', backPage);

    $page.page().trigger('create');
    options.dataUrl = '#item_' + section + '_' + category + '_' + item;
    $.mobile.changePage($page, options);
}


// Add the audio playback controls if the item contains audio (meditation or a book)
function getMediaControlMarkup(itemObj) {
    var mediaControls = '';

    if (itemObj.audio) {
        mediaTimer = setInterval(mediaUpdate, 500);

        mediaControls += '<div class="audio-controls"><div class="ui-grid-a">';
        mediaControls += '<div class="ui-block-a"><a id="play" href="#"><img id="playpauseImage" src="images/playButton.png" alt="Play / Pause"></a></div>';
        mediaControls += '<div class="ui-block-b"><a id="stop" href="#"><img id="stopImage" src="images/stopButton.png" alt="Stop"></a></div>';
        mediaControls += '</div>';
        mediaControls += '<input type="range" name="audio-slider" id="audio-slider" data-highlight="true" data-theme="a" value="0" min="0" max="100" /></div>';
    }

    return mediaControls;
}


function getImageMarkup(itemObj) {
    var image = '';

    var shortFileName = extractFilename(itemObj.image);
    var imageFullPath = window.localStorage.getItem(shortFileName);
    if (imageFullPath === null) imageFullPath = 'images/default.png'; // image not downloaded yet, use a local placeholder

    image = '<img src="' + imageFullPath + '">';
    if (itemObj.video) {
        image = '<div video="true" class="meditation_image ui-block-a">' + image + ' </div>';
    }
    else {
        image = '<div class="meditation_image ui-block-a">' + image + ' </div>';
    }
    return image;
}


function getBookMarkup(itemObj) {
    var book = '';
    bookList = [];
    totalBookSize = 0;
    if (itemObj.audio && Array.isArray(itemObj.audio)) {
        book += '<strong>Sections :</strong>';
        book += '<ul">';
        var audios = itemObj.audio;
        for (var i = 0; i < audios.length; i++) {
            book += '<li class="chapter-list" id="book-playing-' + i + '"><a href="#"><span id="book-icon-' + i + '"> </span>' + audios[i].desc + ' <span id="book-download-' + i + '"></span></a></li>';            
        }
        book += '</ul>';
        book += '<div class="feedbacktitle"><p><a id="audio_clear" data-transition="none" href="#"><img src="images/rateicon.png"> <span id="audio-downloaded">No chapters downloaded</span></a></p></div>';
    }

    return book;
}


// For each chapter item, mark whether it has been downloaded or not
var bookList = []; // array of filenames of the book
var totalBookSize = 0; // size of all chapters in the book

function setBookDownloads(itemObj) {
    //clearAudio = '<a id="audio_clear" data-transition="none" href="#"><span id="audio-downloaded-' + num + '">downloading...</span></a>';
    var audios = itemObj.audio;
    totalBookSize = 0;
    bookList = [];
    for (var i = 0; i < audios.length; i++) {
        var shortFileName = extractFilename(audios[i].url);
        var fullPath = window.localStorage.getItem(shortFileName);
        if (fullPath == null) {
            //$('#book-download-' + i).html(' <div class="tap">(Tap to download)</div>');
            //$('#book-icon-' + i).html('<img src="images/empty-icon.png">');
            $('#book-icon-' + i).html('<img src="images/download-icon.png">');
        }
        else {
            var size = Number(window.localStorage.getItem(shortFileName + '-size'));
            var filesize = toMb(size);
            totalBookSize = totalBookSize + size;
            bookList.push(shortFileName);
            $('#book-icon-' + i).html('<img src="images/play-icon.png">');
            //$('#book-download-' + i).html(' (' + filesize + 'Mb)'); // Do we care about knowing this?
        }
    }
    if (totalBookSize > 0) $('#audio-downloaded').html('remove all chapters (' + toMb(totalBookSize) + 'Mb)');
}


function getVideoMarkup(itemObj) {
    var video = '';
    if (itemObj.video && isNetworkConnected()) {
        var videoLink = itemObj.video;
        var link = videoLink.match(/\?v=(.*)/i);
        video = '<iframe id="videoplayer" width="100%" src="http://www.youtube.com/embed/' + link[1] + '?enablejsapi=1' + '" frameborder="0" allowfullscreen></iframe>';
    }
    return video;
}


function getInformationMarkup(itemObj) {
    var info = '';
    info += '<div class="heading ui-grid-solo"><strong>' + itemObj.name + '</strong></div>';
    info += '<div class="ui-grid-a">';
    var date = new Date(itemObj.date);
    var locale = window.navigator.userLanguage || window.navigator.language;
    info += '<div class="heading ui-block-a">' + date.toLocaleDateString(locale) + '</div>';
    info += '<div class="heading ui-block-b">';
    if (itemObj.audio) {
        //if (myMedia)
        info += '<div id="audio_duration"></div>';
        //else
        //	info += '<div id="audio_duration"><img src="images/ajax-loader.gif"></div>';
    }
    info += '</div>';
    info += '</div>';
    var newDescription = (itemObj.description).replace(/\r\n/gi, "<br/>");
    info += '<div class="info ui-grid-solo"><p>' + newDescription + '</p></div>';
    return info;
}


var audioToDelete = null;

// Remove the downloaded file. Should only be called from the audio page for that file.
function clearAudio(fileURL) {
    if (fileURL !== null) {
        audioToDelete = extractFilename(fileURL);
        navigator.notification.confirm
            (
            'Remove the downloaded meditation?', // message
            onConfirm,            // callback to invoke with index of button pressed
            'Meditation Lounge',           // title
            Array('Yes', 'No')         // buttonLabels
            );
    }
    return false;
}

// Clear all audios in a book
function clearBookAudio() {
    console.log('Delete chapters ' + bookList);

    if (bookList.length == 0) return;

    navigator.notification.confirm
        (
        'Remove the downloaded chapters?', // message
        onChapterConfirm,            // callback to invoke with index of button pressed
        'Meditation Lounge',           // title
        Array('Yes', 'No')         // buttonLabels
        );
    return false;
}


// Remove all audio chapters if user said yes
function onChapterConfirm(buttonIndex) {
    if (buttonIndex == 1) {
        for (var item = 0; item < bookList.length; item++) {
            console.log('Deleting ' + bookList[item]);
            window.localStorage.removeItem(bookList[item]);
            baseDir.getFile(bookList[item], null, onFoundFileToDelete, onErrorHandler);
        }
        onBackButton();
    }
}


// process the confirmation dialog result and call delete if required
function onConfirm(buttonIndex) {
    if (audioToDelete !== null && buttonIndex == 1) {
        console.log('Deleting ' + baseDir.toURL() + ' ' + audioToDelete);
        window.localStorage.removeItem(audioToDelete);
        onBackButton();
        baseDir.getFile(audioToDelete, null, onFoundFileToDelete, onErrorHandler);
    }
}


// Try to delete
function onFoundFileToDelete(file) {
    console.log('Trying to delete ' + file.name);
    file.remove(audioDeletedOK, onErrorHandler);
}


// Deleted OK - finally!
function audioDeletedOK() {
    console.log('Audio file deleted : ' + audioToDelete);
    audioToDelete = null;
}


// Return a file size measured in bytes as Mb. Show 2dp if less than 10Mb
function toMb(size) {
    var value = size / (1024 * 1024);
    return value.toFixed(1);
}


//Return a time in seconds as minutes and seconds
function toTime(time) {
    if (time < 10) return ''; // ignore time until it is sensible.

    var minutes = time / 60.0;
    var str = '';

    if (minutes > 0) str = Math.floor(minutes) + ' min ';
    str += Math.floor((minutes - Math.floor(minutes)) * 60) + ' sec';

    return (str);
}


// Handle the audio playing controls
var myMedia;
var mediaTimer;
var playing = false; // true if playing, false otherwise
var refresh = false; // if true, update the seek when the slider value changes
/*
function aquireScreenDim() {
    cordova.require('cordova/plugin/powermanagement').dim(
        function(){
            console.log('Wakelock dim acquired');
        },
        function(){
            console.log('Error acquiring wakelock dim');
        }
    );
}


function acquireScreenLock() {
    cordova.require('cordova/plugin/powermanagement').acquire(
        function(){
            console.log('Wakelock acquired');
        },
        function(){
            console.log('Error acquiring wakelock');
        }
    );
}


function releaseScreenLock() {
    cordova.require('cordova/plugin/powermanagement').release(
        function(){
            console.log('Wakelock release');
        },
        function(){
            console.log('Error releasing wakelock');
       }
    );
}
*/

// Click on the chapter to start playing
var oldChapter = -1;
function chooseChapterAudio(desc, url, chapterNum) {
    if (myMedia) {
        stopAudio();
        myMedia.release();
        myMedia = null;
    }

    var shortFileName = extractFilename(url);
    var fullPath = window.localStorage.getItem(shortFileName);

    console.log('Clicked on ' + chapterNum + ': ' + desc + ' (' + url + ')' + ' prev ' + oldChapter);
    
    // Mark the clicked chapter and un-mark the old one
    if (chapterNum != oldChapter) {
        $('#book-playing-' + chapterNum + ' a').addClass('playing');
        $('#book-playing-' + oldChapter + ' a').removeClass('playing');
        oldChapter = chapterNum;
    }

    if (fullPath === null) // File has not been downloaded yet
    {
        getAudio(remoteBase + '/' + shortFileName, chapterNum);
        $('#book-download-' + chapterNum).html(' (downloading...)');
    }
    else {
        // Check it really is there and download if not
        baseDir.getFile(shortFileName, null,
            function (fileEntry) {
                console.log('yes, audio OK ' + fileEntry.name);
                myMedia = new Media(fileEntry.toInternalURL(), completedAudio, errorAudio);
                setTimeout(playAudio, 500);
            },
            function () {
                console.log('Audio file missing - mark as such and reload');
                window.localStorage.removeItem(shortFileName);
                getAudio(remoteBase + '/' + shortFileName, chapterNum);
            });
    }
}


function playAudio() {
    if (myMedia) {
        if (!playing) {
            console.log('playing file');
            if (device.platform == 'iOS') window.plugins.insomnia.keepAwake();
            myMedia.play();
            document.getElementById('playpauseImage').src = 'images/pauseButton.png';
            playing = true;
            refresh = true;
        }
        else {
            console.log('pausing file');
            if (device.platform == 'iOS') window.plugins.insomnia.allowSleepAgain();
            myMedia.pause();
            document.getElementById('playpauseImage').src = 'images/playButton.png';
            playing = false;
            refresh = false;
        }
    }
}


function stopAudio() {
    if (myMedia) {
        console.log('stopping file');
        if (device.platform == 'iOS') window.plugins.insomnia.allowSleepAgain();
        playing = false;
        refresh = true;
        myMedia.stop();
        var pauseButton = document.getElementById('playpauseImage');
        if (pauseButton) pauseButton.src = 'images/playButton.png';
    }
}


function completedAudio() {
    if (myMedia && playing) {
        console.log('completed file');
        if (device.platform == 'iOS') window.plugins.insomnia.allowSleepAgain();
        playing = false;
        refresh = true;
        document.getElementById('playpauseImage').src = 'images/playButton.png';
    }
}


// Something went wrong
function errorAudio(err) {
    console.log('media error ' + JSON.stringify(err));
    document.getElementById('playpauseImage').src = 'images/playButton.png';
}


// Update the slider whether the audio is playing or not
function mediaUpdate() {
    if (!myMedia) return;
    myMedia.getCurrentPosition(
        function (position) {
            var duration = myMedia.getDuration();
            if (position <= 0) position = 0;
            if (duration <= 0) duration = 1;
            if (position > duration) position = 0;
            var percentage = position / duration * 100;
            //console.log('position ' + position + ' ' + percentage + '%');

            if (playing || refresh) {
                refresh = true;
                $('#audio-slider').val(percentage);
                $('#audio-slider').slider('refresh');
                refresh = false;
            }

            $('#audio_duration').html(toTime(duration));
        },
        function (e) {
            console.log('Error getting position = ' + e);
        }
        );
}


// listens for the slider change - seek to the appropriate position in the media
// Will also be called when media position is checked by timer. Don't do a seek in that case.
$(document).on('change', '#audio-slider',
    function (e) {
        var moveTo = $('#audio-slider').val();
        //console.log('Slider moved: ' +  moveTo + ' = ' + (moveTo * 10.0 * myMedia.getDuration()) + ' of ' + myMedia.getDuration());
        if (myMedia && !refresh) {
            myMedia.seekTo(moveTo * 10.0 * myMedia.getDuration());
        }
    }
    );


//Create a page for feedback for a single item within a category for a section
/* FEEDBACK REMOVED FOR NOW
function showFeedback(section, category, item, options) {
	var $page = $( '#feedback' );
	var $header = $page.children( ':jqmData(role=header)' );
	var $content = $page.children( ':jqmData(role=content)' );

	var categoryObj = meditationData[section][category];
	var itemObj = categoryObj['items'][item];

	console.log('Feedback popup : ' + section + ' ' + category + ' ' + item + ' ' + itemObj.slug);

	var markup = '<div class="not-outer"><div class="meditation">';

	// Display image header
	var image = '<div class="meditation_image">';
	var shortFileName = extractFilename(itemObj.image);
	var fullPath = window.localStorage.getItem(shortFileName);
	if (fullPath == null) fullPath = remoteBase + '/' + itemObj.image;
	image += '<img src="' + fullPath + '">';
	image += '</div>';
	markup += image;

	markup += '<form class="feedbackform" method="post" data-direction="reverse" accept-charset="utf-8" action="' + '#item_' + section + '_' + category + '_' + item + '">';
	markup += '<div class="feedbackform_comment"><strong>' + (itemObj.feedback ? itemObj.feedback : 'Please leave your comments') + '</strong></div>';
	markup += '<label for="feedback_text" class="ui-hidden-accessible">Feedback textbox:</label><textarea id="feedback_text"></textarea>';
	markup += '<label for="feedback_submit" class="ui-hidden-accessible">Submit button:</label><button type="submit" id="feedback_submit">Send feedback</button>';
	markup += '</form>'

	markup += '</div></div>';

	$content.html( markup );

	// Store the feedback and a list of items to send
	$('#feedback_submit').click(function() {
		console.log('Feedback set : ' + $('#feedback_text').val());
		window.localStorage.setItem(itemObj.slug + '-feedback-given', $('#feedback_text').val());
		var toSend = window.localStorage.getItem('feedback-to-send');
		if (toSend == null) {
			window.localStorage.setItem('feedback-to-send', itemObj.slug );
		}
		else {
			window.localStorage.setItem('feedback-to-send', toSend + ', ' + itemObj.slug );
		}
		console.log('To Send : ' + window.localStorage.getItem('feedback-to-send'));
		return true;
	});

	$header.find( 'h1' ).html( 'rate and improve ' + itemObj.name );

	backPage = '#item_' + section + '_' + category + '_' + item;
	$header.find( 'a' ).attr('href', backPage);

	$page.page().trigger( 'create' );
	options.dataUrl = '#feedback_' + section + '_' + category + '_' + item;

	$.mobile.changePage( $page, options );
}
*/


// Handle click events for all external URLs - don't need different versions for each platform now
function handleExternalURLs() {
    // if (device.platform.toUpperCase() === 'ANDROID') {
    //     $(document).on('click', 'a[href^="http"]',
    //         function (e) {
    //             var url = $(this).attr('href');
    //             console.log('Android - external click ' + url);
    //             // window.open(url, '_system');
    //             cordova.InAppBrowser.open(url, '_blank')
    //             e.preventDefault();
    //         }
    //     );
    // }
    // else if (device.platform.toUpperCase() === 'IOS') {
        $(document).on('click', 'a[href^="http"]',
            function (e) {
                var url = $(this).attr('href');
                // console.log('external click ' + url);
                // window.open(url, '_system');
                cordova.InAppBrowser.open(url, '_system', 'location=no,usewkwebview=yes')
                e.preventDefault();
            }
        );
    // }
}


// Listen for any attempts to call changePage().
$(document).on('pagecontainerbeforechange',
    function (e, data) {
        if (typeof data.toPage === 'string') {
            var u = $.mobile.path.parseUrl(data.toPage);

            // Going to the top level
            if (u.hash == '#menu') {
                backPage = null;
            }

            // Going to the 'getting started' or 'about us' pages
            if (u.hash == '#getting-started' || u.hash == '#about-us') {
                backPage = '#menu';
            }

            // Produce a list of categories
            if (u.hash.substr(0, 9) == '#section_') {
                var item = u.hash.split('_');
                console.log('Section : ' + item[1]);
                showCategories(item[1], data.options);
            }

            // Produce a list of items within a single category
            if (u.hash.substr(0, 10) == '#category_') {
                var item = u.hash.split('_');
                console.log('Category : ' + item[1] + ', ' + item[2]);
                showSingleCategory(item[1], item[2], data.options);
            }

            // Produce a page of a single item
            if (u.hash.substr(0, 6) == '#item_') {
                var item = u.hash.split('_');
                console.log('Item : ' + item[1] + ', ' + item[2] + ', ' + item[3]);
                showItem(item[1], item[2], item[3], data.options);
            }

            // Produce a popup page for feedback of a single item
            // FEEDBACK REMOVED FOR NOW
            //if (u.hash.substr(0, 10) == '#feedback_')
            //{
            //	var item = u.hash.split('_');
            //	console.log('Feedback : ' + item[1] + ', ' + item[2] + ', ' + item[3]);
            //	showFeedback(item[1], item[2], item[3], data.options);
            //}
        }
    }
    );
